@@ -2,11 +2,11 @@
 /*
  * http.php
  *
- * @(#) $Header: /sources/phpprintipp/phpprintipp/HTTP/http.php,v 1.1 2005/12/27 22:02:26 harding Exp $
- * Hacked 2005-12-17 by Thomas Harding to allow special applications (currently IPP: Internet Printing Protocol)
+ * @(#) $Header: /sources/phpprintipp/phpprintipp/HTTP/http.php,v 1.6 2006/01/04 17:44:17 harding Exp $
+ * Hacked 2005-12-17 and 2005/12/31 by Thomas Harding to allow special applications (currently IPP: Internet Printing Protocol)
  *
  * Copyright (c) 2001-2005, Manuel Lemos
- * Copyright (c) 2005, Thomas Harding
+ * Copyright (c) 2005-2006, Thomas Harding
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -18,7 +18,7 @@
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
- *     * Neither the name of the Manuel Lemos nor the names of its
+ *     * Neither the name of Manuel Lemos nor the names of its
  *       contributors may be used to endorse or promote products derived from
  *       this software without specific prior written permission.
  * 
@@ -32,7 +32,8 @@
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. * (c) 2001-2005, Manuel Lemos
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ *
  *
  */
 
@@ -45,7 +46,7 @@ class http_class
 
 	var $protocol="http";
 	var $request_method="GET";
-	var $user_agent='httpclient V1.53-IPP (http://tom.harding.free.fr/downloads/printIPP/HTTP/)';
+	var $user_agent='httpclient V1.55-IPP (http://sv.nongnu.org/projects/phpprintipp)';
 	var $authentication_mechanism="";
 	var $user;
 	var $password;
@@ -62,7 +63,6 @@ class http_class
 	var $request_user;
 	var $request_password;
 	var $request_realm;
-    var $request_file = "";
 	var $request_workstation;
 	var $proxy_request_user;
 	var $proxy_request_password;
@@ -115,7 +115,8 @@ class http_class
 		"Dec"=>"12");
 	var $session='';
 	var $connection_close=0;
-    var $memorized_args; // part of hack
+    var $memorized_args; // part of IPP HACK
+    var $request_file; // part of IPP HACK
 
 	/* Private methods - DO NOT CALL */
 
@@ -143,6 +144,11 @@ class http_class
 		}
 	}
 
+	Function CookieEncode($value, $name)
+	{
+		return($name ? str_replace("=", "%25", $value) : str_replace(";", "%3B", $value));
+	}
+
 	Function SetError($error)
 	{
 		return($this->error=$error);
@@ -196,10 +202,10 @@ class http_class
 			}
 			else
 			{
-				if(feof($this->connection))
-					return($this->SetError("reached the end of data while reading from the HTTP server connection"));
-				$data=fgets($this->connection,100);
-			}
+				    if(feof($this->connection))
+					    return($this->SetError("reached the end of data while reading from the HTTP server connection"));
+				    $data=fgets($this->connection,100);
+			}   
 			if(GetType($data)!="string"
 			|| strlen($data)==0)
 			{
@@ -222,18 +228,32 @@ class http_class
 
 	Function PutLine($line)
 	{
+
+        // {{{ HACKED FOR IPP
+        $read = array($this->connection);
+        if(stream_select($read, $write = NULL, $except = NULL, 0))
+            return 1;
+        // }}} END OF HACK
+        
 		if($this->debug)
 			$this->OutputDebug("C $line");
-		if(!fputs($this->connection,$line."\r\n"))
+	if(!fputs($this->connection,$line."\r\n"))
 		{
+
 			$this->SetDataAccessError("it was not possible to send a line to the HTTP server");
 			return(0);
 		}
+
 		return(1);
 	}
 
 	Function PutData(&$data)
 	{
+        // {{{ HACKED For IPP
+        $read = array($this->connection);
+        if(stream_select($read, $write = NULL, $except = NULL, 0))
+            return 1;
+        // }}} END OF HACK
 		if(strlen($data))
 		{
 			if($this->debug)
@@ -866,20 +886,24 @@ class http_class
 					$end=(GetType($input=Key($files))!="string");
 				}
 			}
-            // {{{ Hack by Thomas Harding for IPP (Internet Printing Protocol)
-            elseif(array_key_exists("PostValues",$arguments) && is_array($arguments["PostValues"]) && array_key_exists("Content-Type",$arguments["PostValues"]))
+            
+            // {{{ HACK by Thomas Harding for IPP (Internet Printing Protocol)
+            elseif(array_key_exists("PostValues",$arguments) && is_array($arguments["PostValues"]) && array_key_exists("Content-Type",$arguments["PostValues"]) && $arguments["PostValues"]["Content-Type"] == "application/ipp" )
             {   
                 $this->memorized_args = $arguments["PostValues"];
+                if (array_key_exists ("BodyFile",$arguments))
+                    $this->memorized_args = array_merge($this->memorized_args, array("BodyFile" => $arguments["BodyFile"]));
+                
                 $values=$arguments["PostValues"];
                 $this->request_body="";
                 if (array_key_exists("Data",$values))
                     $this->request_body .= $values["Data"];
-                if (array_key_exists("File",$values))
-                    $this->request_file = $values["File"];
+                if (array_key_exists("BodyFile",$arguments))
+                    $this->body_file = $arguments["BodyFile"];
                 $this->request_headers["Content-Type"]=$values["Content-Type"];
                 $body_length = strlen($values["Data"]);
-                if ($this->request_file && file_exists($this->request_file))
-                    $body_length += filesize($this->request_file);
+                if (isset($this->body_file) && file_exists($this->body_file))
+                    $body_length += filesize($this->body_file);
                 if (array_key_exists("Filetype",$values) && $values["Filetype"] == "TEXT") {
                     $body_length += 1;
                     $added_char = chr(0x0c);
@@ -890,24 +914,24 @@ class http_class
                 $values=$this->memorized_args;
                 
                 $this->request_headers["Content-Type"]=$values["Content-Type"];
-                
+                $body_length = 0; 
                 $this->request_body="";
                 if (array_key_exists("Data",$values)) {
                     $this->request_body .= $values["Data"];
                     $body_length = strlen($values["Data"]);
                     }
-                if (array_key_exists("File",$values)) {
-                    $this->request_file = $values["File"];
-                    $body_length += filesize($this->request_file);
-                    }
-                if ($this->request_file && file_exists($this->request_file))
-                    $body_length+=filesize($this->request_file);
+                if (array_key_exists("BodyFile",$values)) {
+                    $this->body_file = $values["BodyFile"];
+                    if ($this->body_file && file_exists($this->body_file))
+                        $body_length+=filesize($this->body_file);
+                    } 
                 if (array_key_exists("Filetype",$values) && $values["Filetype"] == "TEXT") {
                     $body_length += 1;
                     $added_char = chr(0x0c);
                     }
             }
             // }}}
+
 			elseif(IsSet($arguments["PostValues"]))
 			{
 				$values=$arguments["PostValues"];
@@ -1034,7 +1058,7 @@ class http_class
 				for(Reset($cookies),$cookie=0;$cookie<count($cookies);Next($cookies),$cookie++)
 				{
 					$cookie_name=Key($cookies);
-					$headers[$h].=" ".UrlEncode($cookie_name)."=".UrlEncode($cookies[$cookie_name]["value"]).";";
+					$headers[$h].=" ".$cookie_name."=".$cookies[$cookie_name]["value"].";";
 				}
 			}
 		}
@@ -1080,7 +1104,7 @@ class http_class
 				curl_setopt($this->connection,CURLOPT_TIMEOUT,$this->timeout);
 			curl_setopt($this->connection,CURLOPT_SSL_VERIFYPEER,0);
 			curl_setopt($this->connection,CURLOPT_SSL_VERIFYHOST,0);
-			$request=$this->request."\r\n".implode("\r\n",$headers)."\r\n".$request_body;
+			$request=$this->request."\r\n".implode("\r\n",$headers)."\r\n\r\n".$request_body;
 			curl_setopt($this->connection,CURLOPT_CUSTOMREQUEST,$request);
 			if($this->debug)
 				$this->OutputDebug("C ".$request);
@@ -1094,47 +1118,45 @@ class http_class
 		{
 			if(($success=$this->PutLine($this->request)))
 			{
-                $string = "";
 				for($header=0;$header<count($headers);$header++)
 				{
-					if(!$success=$this->PutLine($headers[$header])) 
-                    break;
+					if(!$success=$this->PutLine($headers[$header]))
+						break;
 				}
-        
-
-                
 				if($success
 				&& ($success=$this->PutLine(""))
 				&& $body_length)
 				{
-					if(strlen($this->request_body)) {
+					if(strlen($this->request_body)) 
+                    // {{{ BEGIN OF IPP HACK
+                        {
 						$success=$this->PutData($this->request_body);
-                        // {{{ hacked to allow file streaming for IPP
-                        if ($success && $this->request_file) {
-					        if(!($file=@fopen($this->request_file,"rb")))
+                        if ($success && isset($this->body_file)) 
+                            {
+					        if(!($file=@fopen($this->body_file,"rb")))
 								{
-									$this->SetPHPError(sprintf(_("could not open upload file %s"),$this->request_file), $php_errormsg);
-									$success=0;
+								$this->SetPHPError(sprintf(_("could not open upload file %s"),$this->body_file), $php_errormsg);
+								$success=0;
 								}
-                                echo "<pre>";
                                 $i = 0;
    								while(!feof($file) && $success)
-								{
+								    {
                                     $i++;
 									if(GetType($block=@fread($file,$this->file_buffer_length))!="string")
-									{
+									    {
 										$this->SetPHPError(_("could not read upload file"), $php_errormsg);
 										$success=0;
 										break;
-									}
+									    }
 									if(!($success=$this->PutData($block)))
 										break;
-								}
-                                $success = $this->PutData($added_char);
+								    }
+                                if($success) 
+                                    $success = $this->PutData($added_char);
 								fclose($file);
-                        }
-                        // }}}
+                            }
                     }
+                    // }}} END OF HACK
 					else
 					{
 						for($part=0;$part<count($post_parts);$part++)
@@ -1168,11 +1190,17 @@ class http_class
 							if(!($success=$this->PutLine("")))
 								break;
 						}
-						if(!($success=$this->PutLine("--".$boundary."--")))
-							break;
+						if($success)
+							$success=$this->PutLine("--".$boundary."--");
 					}
-					if($success)
-						$sucess=$this->FlushData();
+                    // {{{ HACKED FOR IPP
+                    $read = array($this->connection);
+					if($success )
+                        if(!stream_select($read, $write = NULL, $except = NULL, 0))
+						   $sucess=$this->FlushData();
+					//if($success )
+						   //$sucess=$this->FlushData();
+                    // }}} END OF HACK    
 				}
 			}
 		}
@@ -1182,7 +1210,7 @@ class http_class
 		return("");
 	}
 
-	Function SetCookie($name, $value, $expires="" , $path="/" , $domain="" , $secure=0)
+	Function SetCookie($name, $value, $expires="" , $path="/" , $domain="" , $secure=0, $verbatim=0)
 	{
 		if(strlen($this->error))
 			return($this->error);
@@ -1197,6 +1225,11 @@ class http_class
 		$domain=strtolower($domain);
 		if(!strcmp($domain[0],"."))
 			$domain=substr($domain,1);
+		if(!$verbatim)
+		{
+			$name=$this->CookieEncode($name,1);
+			$value=$this->CookieEncode($name,0);
+		}
 		$this->cookies[$secure][$domain][$path][$name]=array(
 			"name"=>$name,
 			"value"=>$value,
@@ -1277,8 +1310,8 @@ class http_class
 							$cookie_headers=array($headers[$header_name]);
 						for($cookie=0;$cookie<count($cookie_headers);$cookie++)
 						{
-							$cookie_name=trim(UrlDecode($this->Tokenize($cookie_headers[$cookie],"=")));
-							$cookie_value=UrlDecode($this->Tokenize(";"));
+							$cookie_name=trim($this->Tokenize($cookie_headers[$cookie],"="));
+							$cookie_value=$this->Tokenize(";");
 							$domain=$this->request_host;
 							$path="/";
 							$expires="";
@@ -1308,7 +1341,7 @@ class http_class
 										break;
 								}
 							}
-							if(strlen($this->SetCookie($cookie_name, $cookie_value, $expires, $path , $domain, $secure)))
+							if(strlen($this->SetCookie($cookie_name, $cookie_value, $expires, $path , $domain, $secure, 1)))
 								$this->error="";
 						}
 					}
